// Copyright 1997 The Open Group Research Institute.  All rights reserved.

package krb5.lib.crypto;

import krb5.lib.*;

public class DesCbcCrcEType extends DesCbcEType {

	public DesCbcCrcEType() {
	}

	public int eType() {
		return Krb5.ETYPE_DES_CBC_CRC;
	}

	public int minimumPadSize() {
		return 4;
	}

	public int confounderSize() {
		return 8;
	}

	public int checksumType() {
		return Krb5.CKSUMTYPE_CRC32;
	}

	public int checksumSize() {
		return 4;
	}

	public byte[] encrypt(byte[] data, byte[] key) {
		long[] key_sched = new long[16];
		des.des_set_key(key, key_sched);
		int new_size = data.length + confounderSize() +	checksumSize();
		byte[] new_data = new byte[new_size +
			(new_size % blockSize() == 0 ? 0 : blockSize() -
			(new_size % blockSize()))];
		byte[] conf = Confounder.bytes(confounderSize());
		System.arraycopy(conf, 0, new_data, 0, confounderSize());
		System.arraycopy(data, 0, new_data, startOfData(), data.length);
		byte[] cksum = calculateChecksum(new_data, new_data.length);
		System.arraycopy(cksum, 0, new_data, startOfChecksum(),
			checksumSize());
		byte[] cipher = new byte[new_data.length];
		//XXX would like it to work in place
		des.cbc_encrypt(new_data, cipher, key_sched, key, true);
		return cipher;
	}

	public void decrypt(byte[] cipher, byte[] key)
		throws KrbApErrException {
		long[] key_sched = new long[16];
		des.des_set_key(key, key_sched);
		byte[] data = new byte[cipher.length];
		//XXX would like it to work in place
		des.cbc_encrypt(cipher, data, key_sched, key, false);
		System.arraycopy(data, 0, cipher, 0, cipher.length);
		if (!isChecksumValid(data))
			throw new KrbApErrException(Krb5.KRB_AP_ERR_BAD_INTEGRITY);
	}

	public byte[] calculateChecksum(byte[] data, int size) {
		return crc32.byte2crc32sum_bytes(data, size);
	}

}
